Option Explicit On 
Option Strict On
Imports System
Imports System.IO
Imports System.Windows.Forms
Imports System.Drawing
Imports System.Drawing.Imaging
Imports System.Drawing.Printing
Imports System.Text
Imports System.Net 'For MapPoint Web Service network credentials

Public Class TerraService
	Inherits System.Windows.Forms.Form

	'See PublicVars.vb for public variables and procedures

	'You must add a Web reference named MP to the MapPoint staging server's URL
	'before you can build or run this project.
	'You receive the MapPoint Web Service URL, UserID and Password when Microsoft 
	'validates your MapPoint developer agreement or subscription. Replace the
	'following placeholders in PublicVars.vb with your credentials:
	'Public Const strMPUserID As String = "Your MapPoint UserID"
	'Public Const strMPPassword As String = "Your MapPoint Password"

	Private bmpImage As Bitmap
	Private strIE As String = "\Program Files\Internet Explorer\IExplore.exe"
	'Web service types
	Private tsScale As New TS.Scale
	Private tsTheme As New TS.Theme
	Private tiImage As New TS.ThemeInfo
	Private pixFormat As PixelFormat = PixelFormat.Format24bppRgb
	Private strScale As String = "16 mm/px"
	Private frmTSHelp As Form
	Private strLastTitle As String

	'Centerpoint coordinates when navigating
	Private dblLatNav As Double = 0
	Private dblLonNav As Double = 0

	'Array of start and end points for drawing tile boundry gridlines
	Private aptfLinesHor(1, 1) As PointF
	Private aptfLinesVert(1, 1) As PointF

	'Variables for toggling between new and saved images
	Private blnIsTopoMap As Boolean
	Private blnIsMapPoint As Boolean
	Private strFilePath As String = Application.StartupPath
	Private strPhotoFile As String = Application.StartupPath + "\TempPhoto.jpg"
	Private strTopoFile As String = Application.StartupPath + "\TempTopo.jpg"
	Private strMapPointFile As String = Application.StartupPath + "\TempMapPoint.jpg"
	Private blnPhotoFile As Boolean
	Private blnTopoFile As Boolean
	Private blnMapPointFile As Boolean
	Private blnIsFromCities As Boolean

	'Windows Image Acquisition (WIA) Automation Layer GUID constants
	Private wiaJPEG As String = "B96B3CAE-0728-11D3-9D7B-0000F81EF32E"
	Private wiaGIF As String = "B96B3CB0-0728-11D3-9D7B-0000F81EF32E"
	'Following aren't used by USGS (for reference)
	Private wiaBMP As String = "B96B3CAB-0728-11D3-9D7B-0000F81EF32E"
	Private wiaPNG As String = "B96B3CAF-0728-11D3-9D7B-0000F81EF32E"
	Private wiaTIFF As String = "B96B3CB1-0728-11D3-9D7B-0000F81EF32E"

	'Printing variable
	Private fntPrint As Font

#Region " Windows Form Designer generated code "

	Public Sub New()
		MyBase.New()

		'This call is required by the Windows Form Designer.
		InitializeComponent()

		'Add any initialization after the InitializeComponent() call

	End Sub

	'Form overrides dispose to clean up the component list.
	Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
		If disposing Then
			If Not (components Is Nothing) Then
				components.Dispose()
			End If
		End If
		MyBase.Dispose(disposing)
	End Sub

	'Required by the Windows Form Designer
	Private components As System.ComponentModel.IContainer

	'NOTE: The following procedure is required by the Windows Form Designer
	'It can be modified using the Windows Form Designer.  
	'Do not modify it using the code editor.
	Friend WithEvents mnuMain As System.Windows.Forms.MainMenu
	Friend WithEvents mnuFile As System.Windows.Forms.MenuItem
	Friend WithEvents mnuCity As System.Windows.Forms.MenuItem
	Friend WithEvents mnuCitySFO As System.Windows.Forms.MenuItem
	Friend WithEvents mnuCityOAK As System.Windows.Forms.MenuItem
	Friend WithEvents mnuCitySDX As System.Windows.Forms.MenuItem
	Friend WithEvents mnuScale As System.Windows.Forms.MenuItem
	Friend WithEvents mnuScale1 As System.Windows.Forms.MenuItem
	Friend WithEvents mnuScale2 As System.Windows.Forms.MenuItem
	Friend WithEvents mnuScale4 As System.Windows.Forms.MenuItem
	Friend WithEvents mnuScale8 As System.Windows.Forms.MenuItem
	Friend WithEvents mnuScale16 As System.Windows.Forms.MenuItem
	Friend WithEvents mnuScale32 As System.Windows.Forms.MenuItem
	Friend WithEvents mnuCityLAX As System.Windows.Forms.MenuItem
	Friend WithEvents mnuScale64 As System.Windows.Forms.MenuItem
	Friend WithEvents mnuScale128 As System.Windows.Forms.MenuItem
	Friend WithEvents mnuScale256 As System.Windows.Forms.MenuItem
	Friend WithEvents mnuTheme As System.Windows.Forms.MenuItem
	Friend WithEvents mnuThemePhoto As System.Windows.Forms.MenuItem
	Friend WithEvents mnuThemeTopo As System.Windows.Forms.MenuItem
	Friend WithEvents mnuFileSave As System.Windows.Forms.MenuItem
	Friend WithEvents mnuFileExit As System.Windows.Forms.MenuItem
	Friend WithEvents mnuHelp As System.Windows.Forms.MenuItem
	Friend WithEvents mnuHelpAbout As System.Windows.Forms.MenuItem
	Friend WithEvents dlgFileSave As System.Windows.Forms.SaveFileDialog
	Friend WithEvents mnuCityNYC As System.Windows.Forms.MenuItem
	Friend WithEvents mnuTS As System.Windows.Forms.MenuItem
	Friend WithEvents mnuTSHome As System.Windows.Forms.MenuItem
	Friend WithEvents mnuTSWebServices As System.Windows.Forms.MenuItem
	Friend WithEvents mnuPixel As System.Windows.Forms.MenuItem
	Friend WithEvents mnuPixel24RGB As System.Windows.Forms.MenuItem
	Friend WithEvents mnuPixel16RGB As System.Windows.Forms.MenuItem
	Friend WithEvents lblInit As System.Windows.Forms.Label
	Friend WithEvents mnuPlaces As System.Windows.Forms.MenuItem
	Friend WithEvents mnuPlacesSel As System.Windows.Forms.MenuItem
	Friend WithEvents mnuPlacesNew As System.Windows.Forms.MenuItem
	Friend WithEvents mnuFileImageProps As System.Windows.Forms.MenuItem
	Friend WithEvents mnuFileOpen As System.Windows.Forms.MenuItem
	Friend WithEvents dlgFileOpen As System.Windows.Forms.OpenFileDialog
	Friend WithEvents lblVert As System.Windows.Forms.Label
	Friend WithEvents lblHoriz As System.Windows.Forms.Label
	Friend WithEvents mnuCitySEA As System.Windows.Forms.MenuItem
	Friend WithEvents mnuCityMSFT As System.Windows.Forms.MenuItem
	Friend WithEvents mnuThemeGetDates As System.Windows.Forms.MenuItem
	Friend WithEvents dlgPrint As System.Windows.Forms.PrintDialog
	Friend WithEvents mnuFilePrintImage As System.Windows.Forms.MenuItem
	Friend WithEvents mnuThemeGridlines As System.Windows.Forms.MenuItem
	Friend WithEvents mnuNav As System.Windows.Forms.MenuItem
	Friend WithEvents mnuNavNorth As System.Windows.Forms.MenuItem
	Friend WithEvents mnuNavEast As System.Windows.Forms.MenuItem
	Friend WithEvents mnuNavSouth As System.Windows.Forms.MenuItem
	Friend WithEvents mnuNavWest As System.Windows.Forms.MenuItem
	Friend WithEvents mnuNavOrigin As System.Windows.Forms.MenuItem
	Friend WithEvents mnuLandmarks As System.Windows.Forms.MenuItem
	Friend WithEvents mnuLandmarksShow As System.Windows.Forms.MenuItem
	Friend WithEvents pbTiledImage As System.Windows.Forms.ProgressBar
	Friend WithEvents mnuHelpOnline As System.Windows.Forms.MenuItem
	Friend WithEvents mnuThemeMapPoint As System.Windows.Forms.MenuItem
	Friend WithEvents mnuTSMP As System.Windows.Forms.MenuItem
	Friend WithEvents mnuTSMPWS As System.Windows.Forms.MenuItem
	Friend WithEvents mnuThemeDistances As System.Windows.Forms.MenuItem
	Friend WithEvents mnuAddress As System.Windows.Forms.MenuItem
	Friend WithEvents mnuAddressEnter As System.Windows.Forms.MenuItem
	Friend WithEvents mnuThemeSaveFiles As System.Windows.Forms.MenuItem
	<System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
		Dim resources As System.Resources.ResourceManager = New System.Resources.ResourceManager(GetType(TerraService))
		Me.mnuMain = New System.Windows.Forms.MainMenu
		Me.mnuFile = New System.Windows.Forms.MenuItem
		Me.mnuFileOpen = New System.Windows.Forms.MenuItem
		Me.mnuFileSave = New System.Windows.Forms.MenuItem
		Me.mnuFilePrintImage = New System.Windows.Forms.MenuItem
		Me.mnuFileImageProps = New System.Windows.Forms.MenuItem
		Me.mnuFileExit = New System.Windows.Forms.MenuItem
		Me.mnuCity = New System.Windows.Forms.MenuItem
		Me.mnuCitySFO = New System.Windows.Forms.MenuItem
		Me.mnuCityOAK = New System.Windows.Forms.MenuItem
		Me.mnuCityLAX = New System.Windows.Forms.MenuItem
		Me.mnuCitySDX = New System.Windows.Forms.MenuItem
		Me.mnuCitySEA = New System.Windows.Forms.MenuItem
		Me.mnuCityMSFT = New System.Windows.Forms.MenuItem
		Me.mnuCityNYC = New System.Windows.Forms.MenuItem
		Me.mnuTheme = New System.Windows.Forms.MenuItem
		Me.mnuThemePhoto = New System.Windows.Forms.MenuItem
		Me.mnuThemeTopo = New System.Windows.Forms.MenuItem
		Me.mnuThemeMapPoint = New System.Windows.Forms.MenuItem
		Me.mnuThemeGridlines = New System.Windows.Forms.MenuItem
		Me.mnuThemeDistances = New System.Windows.Forms.MenuItem
		Me.mnuThemeGetDates = New System.Windows.Forms.MenuItem
		Me.mnuScale = New System.Windows.Forms.MenuItem
		Me.mnuScale1 = New System.Windows.Forms.MenuItem
		Me.mnuScale2 = New System.Windows.Forms.MenuItem
		Me.mnuScale4 = New System.Windows.Forms.MenuItem
		Me.mnuScale8 = New System.Windows.Forms.MenuItem
		Me.mnuScale16 = New System.Windows.Forms.MenuItem
		Me.mnuScale32 = New System.Windows.Forms.MenuItem
		Me.mnuScale64 = New System.Windows.Forms.MenuItem
		Me.mnuScale128 = New System.Windows.Forms.MenuItem
		Me.mnuScale256 = New System.Windows.Forms.MenuItem
		Me.mnuPixel = New System.Windows.Forms.MenuItem
		Me.mnuPixel24RGB = New System.Windows.Forms.MenuItem
		Me.mnuPixel16RGB = New System.Windows.Forms.MenuItem
		Me.mnuNav = New System.Windows.Forms.MenuItem
		Me.mnuNavNorth = New System.Windows.Forms.MenuItem
		Me.mnuNavEast = New System.Windows.Forms.MenuItem
		Me.mnuNavSouth = New System.Windows.Forms.MenuItem
		Me.mnuNavWest = New System.Windows.Forms.MenuItem
		Me.mnuNavOrigin = New System.Windows.Forms.MenuItem
		Me.mnuPlaces = New System.Windows.Forms.MenuItem
		Me.mnuPlacesSel = New System.Windows.Forms.MenuItem
		Me.mnuPlacesNew = New System.Windows.Forms.MenuItem
		Me.mnuLandmarks = New System.Windows.Forms.MenuItem
		Me.mnuLandmarksShow = New System.Windows.Forms.MenuItem
		Me.mnuAddress = New System.Windows.Forms.MenuItem
		Me.mnuAddressEnter = New System.Windows.Forms.MenuItem
		Me.mnuTS = New System.Windows.Forms.MenuItem
		Me.mnuTSHome = New System.Windows.Forms.MenuItem
		Me.mnuTSWebServices = New System.Windows.Forms.MenuItem
		Me.mnuTSMP = New System.Windows.Forms.MenuItem
		Me.mnuTSMPWS = New System.Windows.Forms.MenuItem
		Me.mnuHelp = New System.Windows.Forms.MenuItem
		Me.mnuHelpOnline = New System.Windows.Forms.MenuItem
		Me.mnuHelpAbout = New System.Windows.Forms.MenuItem
		Me.dlgFileSave = New System.Windows.Forms.SaveFileDialog
		Me.lblInit = New System.Windows.Forms.Label
		Me.dlgFileOpen = New System.Windows.Forms.OpenFileDialog
		Me.lblVert = New System.Windows.Forms.Label
		Me.lblHoriz = New System.Windows.Forms.Label
		Me.pbTiledImage = New System.Windows.Forms.ProgressBar
		Me.dlgPrint = New System.Windows.Forms.PrintDialog
		Me.mnuThemeSaveFiles = New System.Windows.Forms.MenuItem
		Me.SuspendLayout()
		'
		'mnuMain
		'
		Me.mnuMain.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.mnuFile, Me.mnuCity, Me.mnuTheme, Me.mnuScale, Me.mnuPixel, Me.mnuNav, Me.mnuPlaces, Me.mnuLandmarks, Me.mnuAddress, Me.mnuTS, Me.mnuHelp})
		'
		'mnuFile
		'
		Me.mnuFile.Index = 0
		Me.mnuFile.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.mnuFileOpen, Me.mnuFileSave, Me.mnuFilePrintImage, Me.mnuFileImageProps, Me.mnuFileExit})
		Me.mnuFile.Text = "&File"
		'
		'mnuFileOpen
		'
		Me.mnuFileOpen.Index = 0
		Me.mnuFileOpen.Text = "&Open JPEG File ..."
		'
		'mnuFileSave
		'
		Me.mnuFileSave.Enabled = False
		Me.mnuFileSave.Index = 1
		Me.mnuFileSave.Text = "Save Bitmap &As ..."
		'
		'mnuFilePrintImage
		'
		Me.mnuFilePrintImage.Enabled = False
		Me.mnuFilePrintImage.Index = 2
		Me.mnuFilePrintImage.Text = "&Print Image ..."
		'
		'mnuFileImageProps
		'
		Me.mnuFileImageProps.Enabled = False
		Me.mnuFileImageProps.Index = 3
		Me.mnuFileImageProps.Text = "&Image Properties"
		'
		'mnuFileExit
		'
		Me.mnuFileExit.Index = 4
		Me.mnuFileExit.Text = "E&xit"
		'
		'mnuCity
		'
		Me.mnuCity.Index = 1
		Me.mnuCity.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.mnuCitySFO, Me.mnuCityOAK, Me.mnuCityLAX, Me.mnuCitySDX, Me.mnuCitySEA, Me.mnuCityMSFT, Me.mnuCityNYC})
		Me.mnuCity.Text = "&City"
		'
		'mnuCitySFO
		'
		Me.mnuCitySFO.Index = 0
		Me.mnuCitySFO.Text = "&San Francisco"
		'
		'mnuCityOAK
		'
		Me.mnuCityOAK.Index = 1
		Me.mnuCityOAK.Text = "&Oakland"
		'
		'mnuCityLAX
		'
		Me.mnuCityLAX.Index = 2
		Me.mnuCityLAX.Text = "&Los Angeles"
		'
		'mnuCitySDX
		'
		Me.mnuCitySDX.Index = 3
		Me.mnuCitySDX.Text = "&San Diego"
		'
		'mnuCitySEA
		'
		Me.mnuCitySEA.Index = 4
		Me.mnuCitySEA.Text = "Sea&ttle"
		'
		'mnuCityMSFT
		'
		Me.mnuCityMSFT.Index = 5
		Me.mnuCityMSFT.Text = "&Redmond"
		'
		'mnuCityNYC
		'
		Me.mnuCityNYC.Index = 6
		Me.mnuCityNYC.Text = "&New York City"
		'
		'mnuTheme
		'
		Me.mnuTheme.Index = 2
		Me.mnuTheme.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.mnuThemePhoto, Me.mnuThemeTopo, Me.mnuThemeMapPoint, Me.mnuThemeSaveFiles, Me.mnuThemeGridlines, Me.mnuThemeDistances, Me.mnuThemeGetDates})
		Me.mnuTheme.Text = "&Theme"
		'
		'mnuThemePhoto
		'
		Me.mnuThemePhoto.Index = 0
		Me.mnuThemePhoto.Text = "Aerial &Photo"
		'
		'mnuThemeTopo
		'
		Me.mnuThemeTopo.Index = 1
		Me.mnuThemeTopo.Text = "USGS &Topo Map"
		'
		'mnuThemeMapPoint
		'
		Me.mnuThemeMapPoint.Index = 2
		Me.mnuThemeMapPoint.Text = "&MapPoint Map"
		'
		'mnuThemeGridlines
		'
		Me.mnuThemeGridlines.Index = 4
		Me.mnuThemeGridlines.Text = "Draw &Gridlines"
		'
		'mnuThemeDistances
		'
		Me.mnuThemeDistances.Index = 5
		Me.mnuThemeDistances.Text = "S&how Distances"
		'
		'mnuThemeGetDates
		'
		Me.mnuThemeGetDates.Index = 6
		Me.mnuThemeGetDates.Text = "Get Image &Dates"
		'
		'mnuScale
		'
		Me.mnuScale.Index = 3
		Me.mnuScale.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.mnuScale1, Me.mnuScale2, Me.mnuScale4, Me.mnuScale8, Me.mnuScale16, Me.mnuScale32, Me.mnuScale64, Me.mnuScale128, Me.mnuScale256})
		Me.mnuScale.Text = "&Scale"
		'
		'mnuScale1
		'
		Me.mnuScale1.Index = 0
		Me.mnuScale1.Text = "&1 m/px"
		'
		'mnuScale2
		'
		Me.mnuScale2.Index = 1
		Me.mnuScale2.Text = "&2 m/px"
		'
		'mnuScale4
		'
		Me.mnuScale4.Index = 2
		Me.mnuScale4.Text = "&4 m/px"
		'
		'mnuScale8
		'
		Me.mnuScale8.Index = 3
		Me.mnuScale8.Text = "&8 m/px"
		'
		'mnuScale16
		'
		Me.mnuScale16.Index = 4
		Me.mnuScale16.Text = "1&6 m/px"
		'
		'mnuScale32
		'
		Me.mnuScale32.Index = 5
		Me.mnuScale32.Text = "&32 m/px"
		'
		'mnuScale64
		'
		Me.mnuScale64.Index = 6
		Me.mnuScale64.Text = "64 &m/px"
		'
		'mnuScale128
		'
		Me.mnuScale128.Index = 7
		Me.mnuScale128.Text = "128 m/&px"
		'
		'mnuScale256
		'
		Me.mnuScale256.Index = 8
		Me.mnuScale256.Text = "256 m/p&x"
		'
		'mnuPixel
		'
		Me.mnuPixel.Index = 4
		Me.mnuPixel.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.mnuPixel24RGB, Me.mnuPixel16RGB})
		Me.mnuPixel.Text = "&Pixel Format"
		'
		'mnuPixel24RGB
		'
		Me.mnuPixel24RGB.Checked = True
		Me.mnuPixel24RGB.Index = 0
		Me.mnuPixel24RGB.Text = "24 bpp &RGB"
		'
		'mnuPixel16RGB
		'
		Me.mnuPixel16RGB.Index = 1
		Me.mnuPixel16RGB.Text = "16 bpp RG&B (555)"
		'
		'mnuNav
		'
		Me.mnuNav.Enabled = False
		Me.mnuNav.Index = 5
		Me.mnuNav.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.mnuNavNorth, Me.mnuNavEast, Me.mnuNavSouth, Me.mnuNavWest, Me.mnuNavOrigin})
		Me.mnuNav.Text = "&Navigate"
		'
		'mnuNavNorth
		'
		Me.mnuNavNorth.Index = 0
		Me.mnuNavNorth.Text = "&North"
		'
		'mnuNavEast
		'
		Me.mnuNavEast.Index = 1
		Me.mnuNavEast.Text = "&East"
		'
		'mnuNavSouth
		'
		Me.mnuNavSouth.Index = 2
		Me.mnuNavSouth.Text = "&South"
		'
		'mnuNavWest
		'
		Me.mnuNavWest.Index = 3
		Me.mnuNavWest.Text = "&West"
		'
		'mnuNavOrigin
		'
		Me.mnuNavOrigin.Index = 4
		Me.mnuNavOrigin.Text = "&To Origin"
		'
		'mnuPlaces
		'
		Me.mnuPlaces.Index = 6
		Me.mnuPlaces.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.mnuPlacesSel, Me.mnuPlacesNew})
		Me.mnuPlaces.Text = "Places &List"
		'
		'mnuPlacesSel
		'
		Me.mnuPlacesSel.Index = 0
		Me.mnuPlacesSel.Text = "In &Selection"
		'
		'mnuPlacesNew
		'
		Me.mnuPlacesNew.Index = 1
		Me.mnuPlacesNew.Text = "&New Selection"
		'
		'mnuLandmarks
		'
		Me.mnuLandmarks.Enabled = False
		Me.mnuLandmarks.Index = 7
		Me.mnuLandmarks.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.mnuLandmarksShow})
		Me.mnuLandmarks.Text = "&Landmarks"
		'
		'mnuLandmarksShow
		'
		Me.mnuLandmarksShow.Index = 0
		Me.mnuLandmarksShow.Text = "&Show List"
		'
		'mnuAddress
		'
		Me.mnuAddress.Index = 8
		Me.mnuAddress.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.mnuAddressEnter})
		Me.mnuAddress.Text = "&Address"
		'
		'mnuAddressEnter
		'
		Me.mnuAddressEnter.Index = 0
		Me.mnuAddressEnter.Text = "&Enter Address..."
		'
		'mnuTS
		'
		Me.mnuTS.Index = 9
		Me.mnuTS.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.mnuTSHome, Me.mnuTSWebServices, Me.mnuTSMP, Me.mnuTSMPWS})
		Me.mnuTS.Text = "Lin&ks"
		'
		'mnuTSHome
		'
		Me.mnuTSHome.Index = 0
		Me.mnuTSHome.Text = "TerraServer &HomePage"
		'
		'mnuTSWebServices
		'
		Me.mnuTSWebServices.Index = 1
		Me.mnuTSWebServices.Text = "TerraServer &Web Services"
		'
		'mnuTSMP
		'
		Me.mnuTSMP.Index = 2
		Me.mnuTSMP.Text = "&MapPoint HomePage"
		'
		'mnuTSMPWS
		'
		Me.mnuTSMPWS.Index = 3
		Me.mnuTSMPWS.Text = "MapPoint Web &Services"
		'
		'mnuHelp
		'
		Me.mnuHelp.Index = 10
		Me.mnuHelp.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.mnuHelpOnline, Me.mnuHelpAbout})
		Me.mnuHelp.Text = "&Help"
		'
		'mnuHelpOnline
		'
		Me.mnuHelpOnline.Index = 0
		Me.mnuHelpOnline.Text = "Online &Help"
		'
		'mnuHelpAbout
		'
		Me.mnuHelpAbout.Index = 1
		Me.mnuHelpAbout.Text = "&About"
		'
		'dlgFileSave
		'
		Me.dlgFileSave.DefaultExt = "jpg"
		Me.dlgFileSave.Filter = "JPEG files (*.jpg)|*.jpg|PNG files (*.png)|*.png|GIF files (*.gif)|*.gif|BMP file" & _
		"s (*.bmp)|*.bmp"
		Me.dlgFileSave.Title = "Save Current Bitmap File"
		'
		'lblInit
		'
		Me.lblInit.Font = New System.Drawing.Font("Verdana", 9.75!, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, CType(0, Byte))
		Me.lblInit.Location = New System.Drawing.Point(184, 136)
		Me.lblInit.Name = "lblInit"
		Me.lblInit.Size = New System.Drawing.Size(280, 56)
		Me.lblInit.TabIndex = 0
		Me.lblInit.Text = "Left-Click to Load Default Image or Make a City Selection from the Menu (Requires" & _
		" an Internet Connection)"
		'
		'dlgFileOpen
		'
		Me.dlgFileOpen.DefaultExt = "jpg"
		Me.dlgFileOpen.Filter = "JPEG files (*.jpg)|*.jpg"
		Me.dlgFileOpen.Title = "Open Saved Image File"
		'
		'lblVert
		'
		Me.lblVert.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle
		Me.lblVert.Location = New System.Drawing.Point(318, 0)
		Me.lblVert.Name = "lblVert"
		Me.lblVert.Size = New System.Drawing.Size(1, 488)
		Me.lblVert.TabIndex = 1
		Me.lblVert.Visible = False
		'
		'lblHoriz
		'
		Me.lblHoriz.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle
		Me.lblHoriz.Location = New System.Drawing.Point(0, 216)
		Me.lblHoriz.Name = "lblHoriz"
		Me.lblHoriz.Size = New System.Drawing.Size(640, 1)
		Me.lblHoriz.TabIndex = 2
		Me.lblHoriz.Visible = False
		'
		'pbTiledImage
		'
		Me.pbTiledImage.Location = New System.Drawing.Point(184, 432)
		Me.pbTiledImage.Name = "pbTiledImage"
		Me.pbTiledImage.Size = New System.Drawing.Size(272, 16)
		Me.pbTiledImage.Step = 1
		Me.pbTiledImage.TabIndex = 4
		Me.pbTiledImage.Visible = False
		'
		'mnuThemeSaveFiles
		'
		Me.mnuThemeSaveFiles.Index = 3
		Me.mnuThemeSaveFiles.Text = "Save Local &Files"
		'
		'TerraService
		'
		Me.AutoScaleBaseSize = New System.Drawing.Size(5, 13)
		Me.ClientSize = New System.Drawing.Size(640, 480)
		Me.Controls.Add(Me.pbTiledImage)
		Me.Controls.Add(Me.lblHoriz)
		Me.Controls.Add(Me.lblVert)
		Me.Controls.Add(Me.lblInit)
		Me.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog
		Me.Icon = CType(resources.GetObject("$this.Icon"), System.Drawing.Icon)
		Me.MaximizeBox = False
		Me.Menu = Me.mnuMain
		Me.MinimizeBox = False
		Me.Name = "TerraService"
		Me.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen
		Me.Text = "Microsoft Research TerraService and Microsoft MapPoint Web Service Client"
		Me.ResumeLayout(False)

	End Sub

#End Region

	Private Sub TerraService_Load(ByVal sender As Object, _
	  ByVal e As System.EventArgs) Handles MyBase.Load
		'Set menus and check for IE in usual location
		mnuPlacesSel.Enabled = False
		mnuFileSave.Enabled = False
		mnuTheme.Enabled = False
		mnuScale.Enabled = False
		mnuPixel.Enabled = False
		frmTSHelp = New TSHelp
		If File.Exists(strIE) Then
			mnuTS.Enabled = True
		Else
			mnuTS.Enabled = False
		End If

		'Set initial scale and theme values
		tsScale = TS.Scale.Scale16m
		strScale = "16"
		mnuScale16.Checked = True
		mnuThemePhoto.Checked = True

		'Get rid of the cache files, if present
		Call DeleteSavedFiles()

		'Set up label crosshairs (1 pixel labels)
		Dim sizForm As Size = Me.Size
		Dim ptVert As New Point((sizForm.Width \ 2) - 2, 0)
		lblVert.Location = ptVert
		'Compensate for title bar and menu height
		Me.Height = 532
		Dim ptHoriz As New Point(0, (sizForm.Height \ 2) - 27)
		lblHoriz.Location = ptHoriz
	End Sub

	Private Sub TerraService_LeftMouseDown(ByVal sender As Object, _
	 ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseDown
		If e.Button = MouseButtons.Left And bmpImage Is Nothing Then
			'Display the default image
			tsScale = TS.Scale.Scale16m
			strCity = "San Francisco"
			strState = "CA"
			strScale = "16"
			Me.Text = "TerraServer-USA and MapPoint - " + strCity + _
			 ", " + strState + " (Scale = 16 m/px)"
			lblInit.Visible = False
			Call UpdateDisplay(0, 0)
			mnuLandmarks.Enabled = True
		End If
	End Sub

	Private Sub lblInit_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles lblInit.MouseDown
		'Handle click on label
		Call TerraService_LeftMouseDown(sender, e)
	End Sub

	'*************************************
	'Image Creation and Display Procedures
	'*************************************

	Private Sub UpdateDisplay(ByVal dblLatUD As Double, ByVal dblLonUD As Double)
		'Call the GetPlacePoint function with bitmap dimensions in pixels
		'Disable File | Image Properties choice and right-click for Image Data
		'when loaded from temporary photo or topo file
		'dblLatUD and dblLonUD are <> 0 when navigating
		Application.DoEvents()
		Try
			lblInit.Visible = False
			Me.Cursor = Cursors.WaitCursor
			Application.DoEvents()
			If blnIsFromAddress And _
			 Not (blnPhotoFile Or (blnIsTopoMap And blnTopoFile) _
			 Or (blnIsMapPoint And blnMapPointFile)) Then
				'Special condition for addresses
				dblLatUD = dblLatPoint
				dblLonUD = dblLonPoint
			Else
				'Clicked address a second time
				If blnIsMapPoint And blnMapPointFile Then
					blnIsFromAddress = False
				End If
			End If
			Dim bmpFile As Bitmap
			If ((dblLatUD = 0 Or blnIsFromAddress) And blnPhotoFile) And Not _
			 (blnIsTopoMap Or mnuThemeGridlines.Checked _
			 Or mnuThemeDistances.Checked) Then
				bmpFile = GetFileImage(640, 480)
			ElseIf (dblLatUD = 0 Or blnIsFromAddress) And blnTopoFile And blnIsTopoMap And Not _
			 (mnuThemeGridlines.Checked Or mnuThemeDistances.Checked) Then
				bmpFile = GetFileImage(640, 480)
			ElseIf blnIsMapPoint Then			' And (dblLatUD = 0 or blnIsFromAddress)
				If blnMapPointFile Then
					bmpFile = GetFileImage(640, 480)
				Else
					If dblLatUD > 0 Then
						bmpFile = GetMapPointImage(dblLatUD, dblLonUD, 640, 480)
					Else
						bmpFile = GetMapPointImage(0, 0, 640, 480)
					End If
				End If
			Else
				If dblLatUD = 0 And InStr(Me.Text, "MapPoint Address") = 0 Then
					bmpFile = GetPlacePoint(640, 480)
				Else
					'Skip GetPlacePoint
					Dim ptCity As New TS.LonLatPt
					If InStr(Me.Text, "MapPoint Address") > 0 Then
						With ptCity
							If dblLatUD > 0 Then
								.Lat = dblLatUD
								.Lon = dblLonUD
							Else
								.Lat = dblLatPoint
								.Lon = dblLonPoint
							End If
						End With
					Else
						With ptCity
							.Lat = dblLatUD
							.Lon = dblLonUD
						End With
					End If
					Dim TS As New TS.TerraService
					TS.Timeout = 30000
					bmpFile = GetTileImage(TS, ptCity, 640, 480)
				End If
				mnuFileImageProps.Enabled = True
				mnuThemeDistances.Enabled = True
			End If
			If Not (bmpImage Is Nothing) Then
				bmpImage.Dispose()
			End If
			bmpImage = bmpFile

			'Causes OnPaint event to fire
			If Not blnIsMapPoint Then
				'Show crosshairs on other maps
				lblVert.Visible = True
				lblHoriz.Visible = True
			End If
			MyBase.Invalidate()

			'Enable/disable menu choices
			mnuFileSave.Enabled = True
			If mnuThemeSaveFiles.Checked Or blnIsMapPoint Then
				mnuPlacesSel.Enabled = False
				If blnIsMapPoint And Not blnMapPointFile Then
					mnuScale.Enabled = True
					mnuPixel.Enabled = True
					If intTiles > 0 Then
						mnuNav.Enabled = True
						mnuNavOrigin.Enabled = True
					Else
						mnuNav.Enabled = False
					End If
				Else
					mnuNav.Enabled = False
					mnuScale.Enabled = False
					mnuPixel.Enabled = False
				End If
				mnuFileImageProps.Enabled = False
				mnuThemeDistances.Checked = False
				mnuThemeDistances.Enabled = False
				mnuThemeGridlines.Checked = False
				mnuThemeGridlines.Enabled = False
				mnuThemeGetDates.Enabled = False
				mnuLandmarks.Enabled = False
			Else
				mnuFileImageProps.Enabled = True
				mnuPlacesSel.Enabled = True
				mnuNav.Enabled = True
				mnuScale.Enabled = True
				mnuPixel.Enabled = True
				mnuThemeDistances.Enabled = True
				mnuThemeGridlines.Enabled = True
				mnuThemeGetDates.Enabled = True
				mnuLandmarks.Enabled = True
				If InStr(Me.Text, "MapPoint Address") > 0 Then
					mnuNavOrigin.Enabled = False
				Else
					mnuNavOrigin.Enabled = True
				End If
			End If
			mnuTheme.Enabled = True

			'Don't save if you want dates or are navigating
			If (dblLatNav = 0 And dblLonNav = 0) And mnuThemeSaveFiles.Checked And _
			 Not mnuThemeGetDates.Checked Then
				'Save the bmpImage to a file
				If blnIsTopoMap Then
					bmpImage.Save(strTopoFile, ImageFormat.Jpeg)
				ElseIf blnIsMapPoint Then
					bmpImage.Save(strMapPointFile, ImageFormat.Jpeg)
				Else
					bmpImage.Save(strPhotoFile, ImageFormat.Jpeg)
				End If
			Else
				Call DeleteSavedFiles()
			End If

		Catch exc As Exception
			Me.Cursor = Cursors.Default
			MessageBox.Show(exc.Message + exc.StackTrace, _
			 "Error in UpdateDisplay Procedure")
		Finally
			Me.Cursor = Cursors.Default
		End Try
	End Sub

	Private Function GetFileImage(ByVal intWidth As Integer, _
	 ByVal intHeight As Integer) As Bitmap
		'Simplified version of GetPlacePoint using persisted bitmap
		'Loads TempPhoto.jpg, TempTopo.jpg, or an opened JPEG file 
		Dim bmpFile As New Bitmap(intWidth, intHeight, pixFormat)
		Dim grfImage As Graphics = Graphics.FromImage(bmpFile)
		Dim imgFile As Image
		Try
			If blnIsTopoMap Then
				If File.Exists(strTopoFile) Then
					imgFile = Image.FromFile(strTopoFile)
				End If
			ElseIf blnIsMapPoint Then
				If File.Exists(strMapPointFile) Then
					imgFile = Image.FromFile(strMapPointFile)
				End If
			Else
				If File.Exists(strPhotoFile) Then
					imgFile = Image.FromFile(strPhotoFile)
				End If
			End If

			'For safety
			If imgFile Is Nothing Then
				Exit Function
			End If

			'Draw the image
			Dim ptfStart As New PointF(0, 0)
			grfImage.DrawImage(imgFile, ptfStart)
			'Release lock on file
			imgFile.Dispose()

			'Manage menus
			mnuFilePrintImage.Enabled = True
			mnuNav.Enabled = False
			mnuLandmarks.Enabled = False
			Return bmpFile
		Catch excImage As Exception
			Dim strMsg As String = "Error displaying image from locally saved file."
			MsgBox(strMsg + excImage.StackTrace, MsgBoxStyle.Exclamation, _
			 "Error Displaying Image")
		Finally
			If Not imgFile Is Nothing Then
				imgFile.Dispose()
			End If
		End Try
	End Function

	Private Function GetPlacePoint(ByVal intWidth As Integer, _
	 ByVal intHeight As Integer) As Bitmap
		'Call the ConvertPlaceToLonLatPt Web method to return
		'latitude and longitude centerpoint of the image
		Try
			'Instantiate the TerraServer proxy class
			'Set timeout to 30 seconds (default is 100)
			Dim TS As New TS.TerraService
			TS.Timeout = 30000

			'Get image description and projection from TileInfo
			'For testing only - run once
			If strDescPhoto Is Nothing Then
				tiImage = TS.GetTheme(tsTheme.Photo)
				strDescPhoto = tiImage.Description
				strProjPhoto = tiImage.ProjectionName
			End If
			If strDescTopo Is Nothing Then
				tiImage = TS.GetTheme(tsTheme.Topo)
				strDescTopo = tiImage.Description
				strProjTopo = tiImage.ProjectionName
			End If

			' Get the latitude and longitude of the requested city
			Dim plCity As New TS.Place
			With plCity
				.City = strCity
				.State = strState
				.Country = "USA"
			End With
			'Return the centerpoint by invoking the ConvertPlaceToLonLatPt Web method
			Dim ptCity As TS.LonLatPt = TS.ConvertPlaceToLonLatPt(plCity)
			Return GetTileImage(TS, ptCity, intWidth, intHeight)
		Catch excTI As Exception
			MsgBox(excTI.Message + excTI.StackTrace, MsgBoxStyle.Exclamation, _
			 "Get PlacePoint Function Error")
		End Try
	End Function

	Private Function GetTileImage(ByVal TS As TS.TerraService, _
	 ByVal ptCity As TS.LonLatPt, ByVal intWidth As Integer, _
	 ByVal intHeight As Integer) As Bitmap
		'Return an AreaBoundingBox by invoking GetAreaFromPt Web method
		'retrieve the required tiles and draw the image
		Try
			'Set up the progress bar
			pbTiledImage.Value = 0
			pbTiledImage.Visible = True
			Application.DoEvents()
			Dim abbImage As New TS.AreaBoundingBox
			If blnIsTopoMap Then
				abbImage = TS.GetAreaFromPt(ptCity, tsTheme.Topo, _
				 tsScale, intWidth, intHeight)
			Else
				abbImage = TS.GetAreaFromPt(ptCity, tsTheme.Photo, _
				 tsScale, intWidth, intHeight)
			End If
			dblLatPoint = ptCity.Lat
			dblLonPoint = ptCity.Lon
			intOffsetX = abbImage.NorthWest.Offset.XOffset
			intOffsetY = abbImage.NorthWest.Offset.YOffset

			'Generate an image that fits the display region (640 x 480)
			Dim bmpTiles As New Bitmap(intWidth, intHeight, pixFormat)
			Dim grfImage As Graphics = Graphics.FromImage(bmpTiles)

			Dim x1 As Integer = abbImage.NorthWest.TileMeta.Id.X
			Dim y1 As Integer = abbImage.NorthWest.TileMeta.Id.Y
			Dim x2 As Integer = abbImage.NorthEast.TileMeta.Id.X
			Dim y2 As Integer = abbImage.SouthWest.TileMeta.Id.Y

			'Total number of tiles for progress bar
			pbTiledImage.Maximum = (x2 - x1 + 1) * (y1 - y2 + 1)

			'Reset upper bounds of gridlines arrays
			ReDim aptfLinesVert(x2 - x1, 1)
			ReDim aptfLinesHor(y1 - y2, 1)

			Dim intTileX As Integer
			Dim intTileY As Integer
			intTiles = 0
			datEarliestTile = Date.Today
			datLatestTile = Date.Today

			'Starting time of image acquisition loops
			lngGetImageTime = Now.Ticks

			Dim x As Integer
			Dim intHor As Integer
			Dim intVert As Integer
			For x = x1 To x2
				'Loop west to east
				Dim y As Integer
				intHor = 0
				For y = y1 To y2 Step -1
					'Loop north to south
					Dim tilID As TS.TileId = abbImage.NorthWest.TileMeta.Id
					If y = y1 And x = x1 Then
						'Get upper left corner
						dblLatNW = abbImage.NorthWest.TileMeta.NorthWest.Lat
						dblLonNW = abbImage.NorthWest.TileMeta.NorthWest.Lon
					ElseIf y = y2 And x = x2 Then
						'Get lower right corner
						dblLatSE = abbImage.SouthEast.TileMeta.SouthEast.Lat
						dblLonSE = abbImage.SouthEast.TileMeta.SouthEast.Lon
					End If
					tilID.X = x
					tilID.Y = y
					Dim tilImage As Image = _
					 Image.FromStream(New MemoryStream(TS.GetTile(tilID)))
					'Get the image by retrieving all tiles specified and 
					'applying X and Y offsets to the tiled image
					grfImage.DrawImage(tilImage, (x - x1) * tilImage.Width - _
					 abbImage.NorthWest.Offset.XOffset, (y1 - y) * _
					 tilImage.Height - abbImage.NorthWest.Offset.YOffset, _
					 tilImage.Width, tilImage.Height)
					'Tile gridlines arrays
					If intVert = 0 Then
						'Set up horizontal tile lines PointF array
						Dim ptfHor As PointF
						ptfHor.Y = (intHor * tilImage.Height) - _
						 abbImage.NorthWest.Offset.YOffset
						ptfHor.X = 0
						aptfLinesHor(intHor, 0) = ptfHor
						ptfHor.X = Me.Width
						aptfLinesHor(intHor, 1) = ptfHor
					End If
					If intHor = 0 Then
						'Set up vertical tile lines PointF array
						Dim ptfVert As PointF
						ptfVert.Y = 0
						ptfVert.X = (intVert * tilImage.Width) - _
						 abbImage.NorthWest.Offset.XOffset
						aptfLinesVert(intVert, 0) = ptfVert
						ptfVert.Y = Me.Height
						aptfLinesVert(intVert, 1) = ptfVert
					End If

					If intHor = 0 And intVert = 0 Then
						'Get the image props (can be enums or WIA GUIDs)
						'Raw format, cover all bases
						Dim strGuid As String = tilImage.RawFormat.ToString
						If tilImage.RawFormat Is ImageFormat.Jpeg Or _
						 InStr(UCase(strGuid), wiaJPEG) > 0 Then
							'Default for topo maps and aerial photos
							strRawFormat = "JPEG"
						ElseIf tilImage.RawFormat Is ImageFormat.Gif Or _
						  InStr(UCase(strGuid), wiaGIF) > 0 Then
							'Used for some topo maps at 8-bpp Indexed
							strRawFormat = "GIF"
						ElseIf tilImage.RawFormat Is ImageFormat.Png Or _
						 InStr(UCase(strGuid), wiaPNG) > 0 Then
							strRawFormat = "PNG"
						ElseIf tilImage.RawFormat Is ImageFormat.Tiff Or _
						 InStr(UCase(strGuid), wiaJPEG) > 0 Then
							strRawFormat = "TIFF"
						ElseIf tilImage.RawFormat Is ImageFormat.Bmp Or _
						 InStr(UCase(strGuid), wiaTIFF) > 0 Then
							strRawFormat = "BMP"
						Else
							strRawFormat = "Unknown"
						End If

						'Pixel format (enum) - only 24-bpp RGB and 8-bpp Indexed 
						'are used, but cover all types
						If tilImage.PixelFormat = PixelFormat.Format8bppIndexed Then
							'Default for aerial photos and some GIF topo maps
							strPixFormat = "8 bpp Indexed"
						ElseIf tilImage.PixelFormat = PixelFormat.Format24bppRgb Then
							'Default for topo maps
							strPixFormat = "24 bpp RGB"
							'Other unused formats follow
						ElseIf tilImage.PixelFormat = PixelFormat.Format4bppIndexed Then
							strPixFormat = "4 bpp Indexed"
						ElseIf tilImage.PixelFormat = PixelFormat.Format32bppRgb Then
							strPixFormat = "32 bpp RGB"
						ElseIf tilImage.PixelFormat = PixelFormat.Format16bppRgb555 Then
							strPixFormat = "16 bpp RGB 555"
						ElseIf tilImage.PixelFormat = PixelFormat.Format16bppRgb565 Then
							strPixFormat = "16 bpp RGB 565"
						ElseIf tilImage.PixelFormat = PixelFormat.Format16bppGrayScale Then
							strPixFormat = "16 bpp Grayscale"
						Else
							strPixFormat = tilImage.PixelFormat.ToString
						End If
					End If
					'Count all tiles
					intTiles += 1
					'Set the progress bar value
					pbTiledImage.Value = intTiles
					'Get tile size
					If x = x1 Then
						intTileY += tilImage.Height
					End If
					If y = y2 Then
						intTileX += tilImage.Width
					End If
					'Free memory
					tilImage.Dispose()

					If mnuThemeGetDates.Checked Then
						'Get the earliest and latest tile dates to
						'display in the Image Data dialog (invoke for each tile)
						Dim tmTile As TS.TileMeta = TS.GetTileMetaFromTileId(tilID)
						If tmTile.TileExists Then
							Dim datTile As DateTime
							'Get the tile image date
							datTile = tmTile.Capture
							If datEarliestTile = Date.Today Then
								'First capture value
								datEarliestTile = datTile
								datLatestTile = datTile
							Else
								'Remaining capture values
								If datTile < datEarliestTile Then
									datEarliestTile = datTile
								ElseIf datTile > datLatestTile Then
									datLatestTile = datTile
								End If
							End If
						End If
					End If
					intHor += 1
				Next y
				intVert += 1
			Next x
			'Get the net time to acquire the imate
			lngGetImageTime = Now.Ticks - lngGetImageTime

			'Test conversion to base64Encoded XSD type 
			'(required for InfoPath image controls)
			Dim stmBitmap As New MemoryStream
			'Save the bmpTiles to a MemoryStream
			'Cover all possibilities
			If strRawFormat = "JPEG" Then
				'Standard format for photos
				bmpTiles.Save(stmBitmap, ImageFormat.Jpeg)
			ElseIf strRawFormat = "GIF" Then
				'Standard format for topos
				bmpTiles.Save(stmBitmap, ImageFormat.Gif)
			ElseIf strRawFormat = "PNG" Then
				bmpTiles.Save(stmBitmap, ImageFormat.Png)
			ElseIf strRawFormat = "TIFF" Then
				bmpTiles.Save(stmBitmap, ImageFormat.Tiff)
			End If

			'Convert the stream to a byte array
			Dim abytBitmap As Byte() = stmBitmap.ToArray
			intImageBytes = abytBitmap.Length
			stmBitmap.Close()
			'Convert the byte array to a base64-encoded string
			Dim strBase64 As String = Convert.ToBase64String(abytBitmap)
			intImageBase64 = Len(strBase64)
			'Free the memory now
			ReDim abytBitmap(0)
			strBase64 = ""
			'Enable menus
			mnuFilePrintImage.Enabled = True
			mnuNav.Enabled = True
			If Val(strScale) < 64 Then
				mnuLandmarks.Enabled = True
			End If
			'Hide the progress bar
			pbTiledImage.Visible = False
			'Return the resulting image
			Return bmpTiles
		Catch exc As Exception
			Me.Cursor = Cursors.Default
			MessageBox.Show(exc.Message + exc.StackTrace, _
			 "Error in GetTileImage Function")
		Finally
			pbTiledImage.Visible = False
		End Try
	End Function

	Private Function GetMapPointImage(ByVal dblLatUD As Double, _
	 ByVal dblLonUD As Double, ByVal intWidth As Integer, _
	 ByVal intHeight As Integer) As Bitmap
		'Retrieve the MapPoint image (requires a MapPoint agreement)
		'Get the lat/lon, if necessary, and display the map
		If strLastTitle <> Me.Text And Not (blnIsFromLandmarks Or blnIsFromAddress Or InStr(Me.Text, "MapPoint Address") > 0) Then
			'If a different view and not from landmarks, 
			'get the centerpoints from TerraService
			Dim TS As New TS.TerraService
			TS.Timeout = 30000

			' Get the latitude and longitude of the requested city
			Dim plCity As New TS.Place
			With plCity
				.City = strCity
				.State = strState
				.Country = "USA"
			End With
			'Return the centerpoint by invoking the ConvertPlaceToLonLatPt Web method
			Dim ptCity As TS.LonLatPt = TS.ConvertPlaceToLonLatPt(plCity)
			If ptCity.Lat <> Nothing Then
				'In case of nothing returned
				dblLatPoint = ptCity.Lat
				dblLonPoint = ptCity.Lon
			End If
		End If
		strLastTitle = Me.Text

		'MapPoint definitions
		Dim bmpMapPoint As New Bitmap(intWidth, intHeight, pixFormat)
		Dim grfImage As Graphics = Graphics.FromImage(bmpMapPoint)
		Dim mwsRender As MP.RenderServiceSoap = New MP.RenderServiceSoap
		Dim crdMapPoint As New NetworkCredential(strMPUserID, strMPPassword)
		mwsRender.Credentials = crdMapPoint
		mwsRender.PreAuthenticate = True

		'Define the view
		Dim amvViews(0) As MP.ViewByScale
		amvViews(0) = New MP.ViewByScale
		With amvViews(0)
			.CenterPoint = New MP.LatLong
			If dblLatUD = 0 Then
				.CenterPoint.Latitude = dblLatPoint
				.CenterPoint.Longitude = dblLonPoint
			Else
				.CenterPoint.Latitude = dblLatUD
				.CenterPoint.Longitude = dblLonUD
			End If
			.MapScale = 60000 * Val(strScale) / 16
		End With

		'Set map options
		Dim moImage As New MP.MapOptions
		With moImage
			.ReturnType = MP.MapReturnType.ReturnImage
			.Format = New MP.ImageFormat
			.Format.Height = intHeight
			.Format.Width = intWidth
		End With

		'Add a pushpin
		Dim ppCenter(0) As MP.Pushpin
		ppCenter(0) = New MP.Pushpin
		ppCenter(0).PinID = "pin0"
		ppCenter(0).IconName = "0"
		ppCenter(0).IconDataSource = "MapPoint.Icons"
		ppCenter(0).LatLong = amvViews(0).CenterPoint

		'Set up the specification object
		Dim mapSpec As New MP.MapSpecification
		With mapSpec
			.Options = moImage
			.Views = amvViews
			.Pushpins = ppCenter
			.DataSourceName = "MapPoint.NA"
		End With
		Dim imgMap As Image
		Try
			'Get the MapPoint image from the staging server
			Dim mpImage As MP.MapImage()
			mpImage = mwsRender.GetMap(mapSpec)
			imgMap = Image.FromStream(New MemoryStream(mpImage(0).MimeData.Bits))

			'Draw the image
			Dim ptfStart As New PointF(0, 0)
			grfImage.DrawImage(imgMap, ptfStart)
			mnuFilePrintImage.Enabled = True
			'Hide crosshairs
			lblVert.Visible = False
			lblHoriz.Visible = False
			Return bmpMapPoint
		Catch excImage As Exception
			Dim strMsg As String = "Error obtaining image from MapPoint Web Service. "
			MsgBox(strMsg + excImage.StackTrace, MsgBoxStyle.Exclamation, _
			 "Error Obtaining MapPoint Image")
		Finally
			If Not imgMap Is Nothing Then
				imgMap.Dispose()
			End If
		End Try
	End Function

	Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
		'Redraws the image every time the form is invalidated
		If Not (bmpImage Is Nothing) Then
			'Redraw the bitmap on the form and 
			'add tile gridlines, if selected
			Try
				e.Graphics.DrawImage(bmpImage, 0, 0, bmpImage.Width, bmpImage.Height)
				Dim penLines As Pen
				If blnIsTopoMap Then
					'Red pen on topo maps
					penLines = New Pen(Color.Red, 2)
				Else
					'White pen on photos
					penLines = New Pen(Color.White, 2)
				End If
				If mnuThemeGridlines.Checked And _
				  (Not (blnIsTopoMap And blnTopoFile) Or _
				  (Not blnIsTopoMap And blnPhotoFile)) Then
					'Draw tile grid lines on image
					'Process the horizontal and vertical line endpoints arrays
					Dim intCtr As Integer
					For intCtr = 0 To UBound(aptfLinesHor, 1)
						'Draw the horizontal lines
						e.Graphics.DrawLine(penLines, aptfLinesHor(intCtr, 0), _
						 aptfLinesHor(intCtr, 1))
					Next
					For intCtr = 0 To UBound(aptfLinesVert, 1)
						'draw the vertical lines
						e.Graphics.DrawLine(penLines, aptfLinesVert(intCtr, 0), _
						 aptfLinesVert(intCtr, 1))
					Next
				End If
				If mnuThemeDistances.Checked And Not blnIsMapPoint Then
					'Get east-west tile distance
					dblTileDistX = GetDistanceKm(dblLatNW, dblLonNW, dblLatNW, dblLonSE)
					'Get north-south tile distance
					dblTileDistY = GetDistanceKm(dblLatNW, dblLonNW, dblLatSE, dblLonNW)
					'Get east-west image distance
					dblImageDistX = CalculateImageDist(dblTileDistX, "X")
					'Get north-south image distance
					dblImageDistY = CalculateImageDist(dblTileDistY, "Y")
					Dim strFormat As String
					If dblImageDistX >= 100 Or dblImageDistY >= 100 Then
						strFormat = "#,##0"
					ElseIf dblImageDistX >= 10 Or dblImageDistY >= 10 Then
						strFormat = "#0.0"
					Else
						strFormat = "#0.00"
					End If
					Dim strHoriz As String = Format(dblImageDistX, strFormat) + " km"
					Dim strVert As String = Format(dblImageDistY, strFormat) + " km"

					Dim penText As New Pen(Color.White, 20)
					If blnIsTopoMap Then
						penText.Color = Color.Red
					End If
					'Draw distance lines, arrows, etc. on the grid
					With e.Graphics
						'Draw interrupted vertical line
						.DrawLine(penLines, 20, 0, 20, 200)
						.DrawLine(penLines, 20, 280, 20, 480)
						If blnIsTopoMap Then
							'Create a red text background
							.DrawLine(penText, 20, 200, 20, 280)
						End If
						'Draw vertical arrows
						.DrawLine(penLines, 20, 0, 27, 10)
						.DrawLine(penLines, 20, 0, 13, 10)
						.DrawLine(penLines, 20, 480, 27, 470)
						.DrawLine(penLines, 20, 480, 13, 470)

						'Draw interrupted horizontal line
						.DrawLine(penLines, 0, 460, 280, 460)
						.DrawLine(penLines, 360, 460, 640, 460)
						If blnIsTopoMap Then
							'Create a white text background
							.DrawLine(penText, 280, 460, 360, 460)
						End If
						'Draw horizontal arrows
						.DrawLine(penLines, 0, 460, 10, 453)
						.DrawLine(penLines, 0, 460, 10, 467)
						.DrawLine(penLines, 640, 460, 630, 453)
						.DrawLine(penLines, 640, 460, 630, 467)

						'Set up the font, brush, and format (from VB.NET drawing samples)
						Dim drawFont As New System.Drawing.Font("Arial", 12, FontStyle.Bold)
						Dim drawBrush As New System.Drawing.SolidBrush(Color.White)
						Dim drawFormat As New System.Drawing.StringFormat
						drawFormat.FormatFlags = StringFormatFlags.DirectionVertical _
						 Or StringFormatFlags.DirectionRightToLeft
						.DrawString(strVert, drawFont, drawBrush, 27, 210, drawFormat)

						'Draw horizontal text
						drawFormat.FormatFlags = Nothing
						.DrawString(strHoriz, drawFont, drawBrush, 290, 450, drawFormat)

						'Get rid of the objects
						penLines.Dispose()
						penText.Dispose()
						drawFont.Dispose()
						drawBrush.Dispose()
						drawFormat.Dispose()
					End With
				End If
			Catch excPaint As Exception
				Dim strMsg As String = "Error in overridden OnPaint event handler."
				MsgBox(strMsg + excPaint.StackTrace, MsgBoxStyle.Exclamation, _
				 "OnPaint Exception")
			End Try
		End If
	End Sub

	'*************************************************
	'Menu Choice Event Handlers and Related Procedures
	'Appear below in left to right main-menu sequence
	'and top to bottom of menu choices
	'*************************************************

	Private Sub UpdateFromMenu()
		'Update the form title bar and call UpdateDisplay
		If InStr(Me.Text, "MapPoint Address") = 0 Or blnIsFromPlaces Or blnIsFromCities Then
			Me.Text = "TerraServer-USA and MapPoint - " + strCity + ", " + _
			 strState + " (Scale = " + strScale + " m/px)"
		End If
		Application.DoEvents()
		mnuFilePrintImage.Enabled = False
		Call UpdateDisplay(dblLatNav, dblLonNav)
	End Sub

	'******************
	'File menu handlers
	'******************

	Private Sub mnuFileOpen_Click(ByVal sender As System.Object, _
	 ByVal e As System.EventArgs) Handles mnuFileOpen.Click
		'Show the File Open dialog
		dlgFileOpen.ShowDialog()
		With dlgFileOpen
			.InitialDirectory = strFilePath
		End With
	End Sub

	Private Sub dlgFileOpen_FileOk(ByVal sender As Object, _
	 ByVal e As System.ComponentModel.CancelEventArgs) Handles dlgFileOpen.FileOk
		'Open and display the file image
		Try
			With dlgFileOpen
				blnPhotoFile = False
				blnTopoFile = False
				blnMapPointFile = False
				blnIsTopoMap = False
				blnIsMapPoint = False
				'Copy the selected file to TempTopo.jpg or TempPhoto.jpg and display it
				If Mid(.FileName, Len(.FileName) - 2, 3) = "jpg" Then
					If Mid(.FileName, InStr(.FileName, ".") - 3, 1) = "T" Then
						'It's a topo file
						File.Copy(.FileName, strFilePath + "\TempTopo.jpg", True)
						blnTopoFile = True
						blnIsTopoMap = True
					ElseIf Mid(.FileName, InStr(.FileName, ".") - 3, 1) = "P" Then
						'It's a photo file
						File.Copy(.FileName, strFilePath + "\TempPhoto.jpg", True)
						blnPhotoFile = True
					ElseIf Mid(.FileName, InStr(.FileName, ".") - 3, 1) = "M" Then
						'It's a MapPoint file
						File.Copy(.FileName, strFilePath + "\TempMapPoint.jpg", True)
						blnMapPointFile = True
						blnIsMapPoint = True
					ElseIf .FileName = strFilePath + "\TempPhoto.jpg" Then
						blnPhotoFile = True
					ElseIf .FileName = strFilePath + "\TempTopo.jpg" Then
						blnTopoFile = True
						blnIsTopoMap = True
					ElseIf .FileName = strFilePath + "\TempMapPoint.jpg" Then
						blnMapPointFile = True
						blnIsMapPoint = True
					End If
					If blnIsTopoMap Then
						Me.Text = "Topo"
					ElseIf blnIsMapPoint Then
						Me.Text = "MapPoint"
					Else
						Me.Text = "Photo"
					End If
					Me.Text += " Image From '" & Mid(.FileName, _
					 InStrRev(.FileName, "\") + 1) & "' File"
					Call UpdateDisplay(0, 0)
					'Disable menus
					mnuTheme.Enabled = False
					mnuScale.Enabled = False
					mnuPixel.Enabled = False
					mnuPlacesSel.Enabled = False
					blnIsFromAddress = False
				Else
					'Shouldn't occur due to filter, but might
					MsgBox("You can open only JPEG files.", _
					 MsgBoxStyle.Exclamation, "Wrong Bitmap Type")
				End If
			End With
		Catch excOpen As Exception
			MsgBox(excOpen.Message + excOpen.StackTrace, _
			 MsgBoxStyle.Exclamation, "Error Opening File")
		End Try
	End Sub

	Private Sub mnuFileSave_Click(ByVal sender As Object, _
	 ByVal e As System.EventArgs) Handles mnuFileSave.Click
		If Not bmpImage Is Nothing Then
			'Open the Save dialog
			With dlgFileSave
				Try
					.InitialDirectory = strFilePath
					Call AbbreviateState()
					If strCity Is Nothing Then
						'From opening a saved file before acquiring an image or map
						Dim strTemp As String = Me.Text
						If InStr(strTemp, "'") > 0 Then
							strTemp = Mid(strTemp, InStr(strTemp, "'") + 1)
							.FileName = Mid(strTemp, 1, InStr(strTemp, ".") - 1)
						Else
							.FileName = ""
						End If
					Else
						.FileName = Replace(strCity, " ", "") + strState + strScale
						If blnIsTopoMap Then
							.FileName = .FileName + "T"
						ElseIf blnIsMapPoint Then
							.FileName = .FileName + "M"
						Else
							.FileName = .FileName + "P"
						End If
						If mnuPixel16RGB.Checked Then
							.FileName = .FileName + "16"
						Else
							.FileName = .FileName + "24"
						End If
					End If
					.ShowDialog()
				Catch excFile As Exception
					MsgBox(excFile.Message + excFile.StackTrace, _
					 MsgBoxStyle.Exclamation, "Error in File Name")
				End Try
			End With
		End If
	End Sub

	Private Sub dlgFileSave_FileOk(ByVal sender As Object, _
	ByVal e As System.ComponentModel.CancelEventArgs) Handles dlgFileSave.FileOk
		If Not bmpImage Is Nothing Then
			'Save the file in the specified format
			Try
				With dlgFileSave
					Dim strExt As String = .FileName
					strExt = Mid(strExt, InStrRev(strExt, "."))
					If UCase(strExt) = ".JPG" Then
						bmpImage.Save(.FileName, ImageFormat.Jpeg)
					ElseIf UCase(strExt) = ".PNG" Then
						bmpImage.Save(.FileName, ImageFormat.Png)
					ElseIf UCase(strExt) = ".GIF" Then
						bmpImage.Save(.FileName, ImageFormat.Gif)
					ElseIf UCase(strExt) = ".BMP" Then
						bmpImage.Save(.FileName, ImageFormat.Bmp)
					Else
						MsgBox("Only JPEG, PNG, GIF, or BMP formats are allowed.", _
						 MsgBoxStyle.Exclamation, "Bitmat Format Error")
					End If
				End With
			Catch excFile As Exception
				MsgBox(excFile.Message + excFile.StackTrace, _
				 MsgBoxStyle.Exclamation, "Error Saving File")
			End Try
		End If
	End Sub

	Private Sub mnuFileImageProps_Click(ByVal sender As System.Object, _
	 ByVal e As System.EventArgs) Handles mnuFileImageProps.Click
		'Display the Image Properties dialog
		frmImageData.ShowDialog()
	End Sub

	Private Sub mnuFileExit_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFileExit.Click
		'Bibi
		End
	End Sub

	'**********************************
	'File | Print Image event handlers
	'**********************************

	Private Sub mnuFilePrintImage_Click(ByVal sender As System.Object, _
	 ByVal e As System.EventArgs) Handles mnuFilePrintImage.Click
		'Print the current image image bitmap and data in landscape 
		'orientation. (Printed image doesn't show gridlines.)
		Try
			fntPrint = New Font("Arial", 10, FontStyle.Bold)
			Dim prtDoc As PrintDocument = New PrintDocument
			Dim prtSet As PrinterSettings = New PrinterSettings
			Dim prtMargins As Margins = New Margins
			With prtMargins
				'Set top, bottom, and left print margins
				'These margins aren't used for printing
				.Top = 50
				.Bottom = 50
				.Left = 75
				.Right = 75
			End With
			With prtSet
				'Limit to 1 page per image
				.Collate = False
				.MaximumPage = 1
				.MinimumPage = 1
			End With
			With prtDoc
				'Specify landscape mode
				.PrinterSettings = prtSet
				.DefaultPageSettings.Landscape = True
				.DefaultPageSettings.Margins = prtMargins
			End With
			'Add the document to the print dialog
			dlgPrint.Document = prtDoc
			If dlgPrint.ShowDialog() = DialogResult.OK Then
				'Set up the PrintPage event handler
				AddHandler prtDoc.PrintPage, _
				 New PrintPageEventHandler(AddressOf Me.PrintPage)
				prtDoc.Print()
			End If
		Catch excPrint As Exception
			Dim strMsg As String = "An error occurred printing the image. "
			MsgBox(strMsg + excPrint.StackTrace, MsgBoxStyle.Exclamation, _
			 "Printing Error")
		End Try
	End Sub

	Private Sub PrintPage(ByVal sender As Object, ByVal ev As PrintPageEventArgs)
		'This event handler is called for each printed page
		'Graphics positions are explicit, not calculated from margins
		Dim strTextLine As String
		Dim intPosX As Integer = 200
		Dim intPosY As Integer = ev.MarginBounds.Top

		Try
			'Draw the form caption 
			strTextLine = Me.Text
			ev.Graphics.DrawString(strTextLine, fntPrint, Brushes.Black, _
			 intPosX, intPosY, New StringFormat)
			'Draw the bitmap image
			Dim ptfLoc As PointF = New PointF
			ptfLoc.X = 200
			ptfLoc.Y = 100
			ev.Graphics.DrawImage(bmpImage, ptfLoc)
			If InStr(strTextLine, ".jpg") = 0 Then
				'Image is not from a saved JPEG file, so image data is available
				'Add the significant image data below the bitmap
				Dim intCol As Integer
				Dim intRow As Integer
				intPosX = 250
				intPosY = 650
				For intCol = 1 To 4
					For intRow = 1 To 3
						Select Case intCol
							Case 1
								If intRow = 1 Then
									strTextLine = "Centerpoint Latitude:"
								ElseIf intRow = 2 Then
									strTextLine = "Centerpoint Longitude:"
								Else
									strTextLine = "Earliest Tile Capture Date:"
								End If
							Case 2
								If intRow = 1 Then
									strTextLine = Format(dblLatPoint, "#0.000")
								ElseIf intRow = 2 Then
									strTextLine = Format(dblLonPoint, "#0.000")
								Else
									strTextLine = datEarliestTile.ToShortDateString
								End If
							Case 3
								If intRow = 1 Then
									strTextLine = "West-East Distance, km:"
								ElseIf intRow = 2 Then
									strTextLine = "North-South Distance, km:"
								Else
									strTextLine = "Latest Tile Capture Date:"
								End If
							Case 4
								If intRow = 1 Then
									dblTileDistX = GetDistanceKm(dblLatNW, dblLonNW, _
									 dblLatNW, dblLonSE)
									strTextLine = Format(CalculateImageDist(dblTileDistX, "X"), _
									 "#0.000")
								ElseIf intRow = 2 Then
									dblTileDistY = GetDistanceKm(dblLatNW, dblLonNW, _
									 dblLatSE, dblLonNW)
									strTextLine = Format(CalculateImageDist(dblTileDistY, "Y"), _
									 "#0.000")
								Else
									strTextLine = datLatestTile.ToShortDateString
								End If
						End Select
						If intRow = 1 Then
							ev.Graphics.DrawString(strTextLine, fntPrint, Brushes.Black, _
							 intPosX, intPosY, New StringFormat)
						ElseIf intRow = 2 Then
							'Move down .030"
							ev.Graphics.DrawString(strTextLine, fntPrint, Brushes.Black, _
							 intPosX, intPosY + 30, New StringFormat)
							If datEarliestTile = Date.Today Or _
							 Not mnuThemeGetDates.Checked Then
								'There are no tile dates for this image
								Exit For
							End If
						Else
							'Move down another .030"
							ev.Graphics.DrawString(strTextLine, fntPrint, Brushes.Black, _
							 intPosX, intPosY + 60, New StringFormat)
						End If
					Next intRow
					If intCol = 1 Or intCol = 3 Then
						'Data caption
						intPosX += 200
					Else
						'Data alue
						intPosX += 100
					End If
				Next intCol
			End If
			ev.HasMorePages = False
		Catch excPrint As Exception
			Dim strMsg As String = "An error occurred in the PrintPage event handler. "
			MsgBox(strMsg + excPrint.StackTrace, MsgBoxStyle.Exclamation, _
			 "Printing Error")
		End Try
	End Sub

	'******************
	'City menu handlers
	'******************

	Private Sub mnuCitySFO_Click(ByVal sender As Object, _
	 ByVal e As System.EventArgs) Handles mnuCitySFO.Click
		mnuThemeTopo.Enabled = True
		strCity = "San Francisco"
		strState = "CA"
		Call CityMenuOps()
	End Sub

	Private Sub mnuCityOAK_Click(ByVal sender As Object, ByVal e As System.EventArgs) _
	  Handles mnuCityOAK.Click
		strCity = "Oakland"
		strState = "CA"
		Call CityMenuOps()
	End Sub

	Private Sub mnuCityLAX_Click(ByVal sender As Object, ByVal e As System.EventArgs) _
	  Handles mnuCityLAX.Click
		mnuThemeTopo.Enabled = True
		'City Hall required to locate center of map downtown
		strCity = "Los Angeles City Hall"
		strState = "CA"
		Call CityMenuOps()
		Me.Text = Replace(Me.Text, strCity, "Los Angeles")
	End Sub

	Private Sub mnuCitySDX_Click(ByVal sender As Object, ByVal e As System.EventArgs) _
	  Handles mnuCitySDX.Click
		mnuThemeTopo.Enabled = True
		'Convention Center required to locate center of map downtown
		strCity = "San Diego Convention Center"
		strState = "CA"
		Call CityMenuOps()
		Me.Text = Replace(Me.Text, strCity, "San Diego")
	End Sub

	Private Sub mnuCitySEA_Click(ByVal sender As System.Object, _
	 ByVal e As System.EventArgs) Handles mnuCitySEA.Click
		mnuThemeTopo.Enabled = True
		strCity = "Seattle"
		strState = "WA"
		Call CityMenuOps()
	End Sub

	Private Sub mnuCityMSFT_Click(ByVal sender As System.Object, _
	 ByVal e As System.EventArgs) Handles mnuCityMSFT.Click
		strCity = "Redmond"
		strState = "WA"
		Call CityMenuOps()
	End Sub

	Private Sub mnuCityNYC_Click(ByVal sender As Object, _
	 ByVal e As System.EventArgs) Handles mnuCityNYC.Click
		mnuThemeTopo.Enabled = True
		strCity = "New York"
		strState = "NY"
		Call CityMenuOps()
	End Sub

	Public Sub CityMenuOps()
		'Common operations for all Cities menus
		Call DeleteSavedFiles()
		mnuThemeTopo.Enabled = True
		dblLatNav = 0
		dblLonNav = 0
		blnIsFromLandmarks = False
		blnIsFromAddress = False
		'Prevent repaint 
		blnIsFromCities = True
		Call UpdateFromMenu()
		blnIsFromCities = False
	End Sub

	'*******************
	'Theme menu handlers
	'*******************

	Private Sub mnuThemePhoto_Click(ByVal sender As Object, _
	 ByVal e As System.EventArgs) Handles mnuThemePhoto.Click
		blnIsMapPoint = False
		blnIsTopoMap = False
		mnuScale1.Enabled = True
		If File.Exists(strPhotoFile) Then
			blnPhotoFile = True
		Else
			blnPhotoFile = False
		End If
		mnuThemePhoto.Checked = True
		mnuThemeTopo.Checked = False
		mnuThemeMapPoint.Checked = False
		blnIsFromAddress = False
		Call UpdateDisplay(dblLatNav, dblLonNav)
	End Sub

	Private Sub mnuThemeTopo_Click(ByVal sender As Object, _
	 ByVal e As System.EventArgs) Handles mnuThemeTopo.Click
		blnIsTopoMap = True
		blnIsMapPoint = False
		mnuScale1.Enabled = False
		If File.Exists(strTopoFile) And Not mnuThemeGridlines.Checked Then
			'Can't save lines on topo maps
			blnTopoFile = True
		Else
			blnTopoFile = False
		End If
		mnuThemePhoto.Checked = False
		mnuThemeTopo.Checked = True
		mnuThemeMapPoint.Checked = False
		blnIsFromAddress = False
		Call UpdateDisplay(dblLatNav, dblLonNav)
	End Sub

	Private Sub mnuThemeMapPoint_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuThemeMapPoint.Click
		'Retrieve the MapPoint topo map
		mnuThemePhoto.Checked = False
		mnuThemeTopo.Checked = False
		mnuThemeMapPoint.Checked = True
		blnIsMapPoint = True
		blnIsTopoMap = False
		If File.Exists(strMapPointFile) Then
			blnMapPointFile = True
		Else
			blnMapPointFile = False
		End If
		blnIsFromAddress = False
		Call UpdateDisplay(0, 0)
	End Sub

	Private Sub mnuThemeGridlines_Click(ByVal sender As System.Object, _
	  ByVal e As System.EventArgs) Handles mnuThemeGridlines.Click
		'Toggle display of gridlines
		With mnuThemeGridlines
			If .Checked Then
				.Checked = False
				Me.Invalidate()
			Else
				.Checked = True
				If Not (blnPhotoFile Or blnTopoFile Or blnMapPointFile _
				 Or bmpImage Is Nothing) Then
					MyBase.Invalidate()
				Else
					Call DeleteSavedFiles()
					Call UpdateDisplay(dblLatNav, dblLonNav)
				End If
			End If
		End With
	End Sub

	Private Sub mnuThemeDistances_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuThemeDistances.Click
		'Toggle Show distances menu choice
		With mnuThemeDistances
			If .Checked Then
				.Checked = False
				lblVert.Visible = True
				lblHoriz.Visible = True
				MyBase.Invalidate()
			Else
				.Checked = True
				If Not (blnIsMapPoint Or bmpImage Is Nothing) Then
					'Add the distance graphics
					lblVert.Visible = False
					lblHoriz.Visible = False
					MyBase.Invalidate()
				End If
			End If
		End With
	End Sub

	Private Sub mnuThemeSaveFiles_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuThemeSaveFiles.Click
		'Toggles saving local files
		With mnuThemeSaveFiles
			If .Checked Then
				.Checked = False
				Call DeleteSavedFiles()
			Else
				.Checked = True
			End If
		End With
	End Sub

	Private Sub mnuThemeGetDates_Click(ByVal sender As System.Object, _
	 ByVal e As System.EventArgs) Handles mnuThemeGetDates.Click
		With mnuThemeGetDates
			If .Checked Then
				.Checked = False
			Else
				Dim strMsg As String = "Obtaining earliest and latest tile dates " + _
				 "increases image acquisition time by about 50%. Do you want to " + _
				 "display these dates in the Image Data dialog?" + vbCrLf + vbCrLf
				strMsg += "You must re-acquire every image to obtain tile dates."
				If MsgBox(strMsg, MsgBoxStyle.Question Or MsgBoxStyle.YesNo, _
				 "Get Image Dates Requested") = MsgBoxResult.Yes Then
					.Checked = True
					'Remove the saved files to enable getting dates
					blnPhotoFile = False
					blnTopoFile = False
					Call DeleteSavedFiles()
					Call UpdateDisplay(dblLatNav, dblLonNav)
				End If
				'Repaint to get rid of partial gridlines
				Me.Invalidate()
			End If
		End With
	End Sub

	'*******************
	'Scale menu handlers
	'*******************

	Private Sub mnuScale1_Click(ByVal sender As Object, _
	 ByVal e As System.EventArgs) Handles mnuScale1.Click
		mnuThemeTopo.Enabled = False
		mnuLandmarks.Enabled = True
		strScale = "1"
		tsScale = TS.Scale.Scale1m
		Call ClearScaleCheck()
		mnuScale1.Checked = True
		Call DeleteSavedFiles()
		Call UpdateFromMenu()
	End Sub

	Private Sub mnuScale2_Click(ByVal sender As Object, _
	 ByVal e As System.EventArgs) Handles mnuScale2.Click
		mnuThemeTopo.Enabled = True
		mnuLandmarks.Enabled = True
		strScale = "2"
		tsScale = TS.Scale.Scale2m
		Call ClearScaleCheck()
		mnuScale2.Checked = True
		Call DeleteSavedFiles()
		Call UpdateFromMenu()
	End Sub

	Private Sub mnuScale4_Click(ByVal sender As Object, _
	 ByVal e As System.EventArgs) Handles mnuScale4.Click
		mnuThemeTopo.Enabled = True
		mnuLandmarks.Enabled = True
		strScale = "4"
		tsScale = TS.Scale.Scale4m
		Call ClearScaleCheck()
		mnuScale4.Checked = True
		Call DeleteSavedFiles()
		Call UpdateFromMenu()
	End Sub

	Private Sub mnuScale8_Click(ByVal sender As Object, _
	 ByVal e As System.EventArgs) Handles mnuScale8.Click
		mnuThemeTopo.Enabled = True
		mnuLandmarks.Enabled = True
		strScale = "8"
		tsScale = TS.Scale.Scale8m
		Call ClearScaleCheck()
		mnuScale8.Checked = True
		Call DeleteSavedFiles()
		Call UpdateFromMenu()
	End Sub

	Private Sub mnuScale16_Click(ByVal sender As Object, _
	 ByVal e As System.EventArgs) Handles mnuScale16.Click
		mnuThemeTopo.Enabled = True
		mnuLandmarks.Enabled = True
		strScale = "16"
		tsScale = TS.Scale.Scale16m
		Call ClearScaleCheck()
		mnuScale16.Checked = True
		Call DeleteSavedFiles()
		Call UpdateFromMenu()
	End Sub

	Private Sub mnuScale32_Click(ByVal sender As Object, _
	 ByVal e As System.EventArgs) Handles mnuScale32.Click
		mnuThemeTopo.Enabled = True
		mnuLandmarks.Enabled = True
		strScale = "32"
		tsScale = TS.Scale.Scale32m
		Call ClearScaleCheck()
		mnuScale32.Checked = True
		Call DeleteSavedFiles()
		Call UpdateFromMenu()
	End Sub

	Private Sub mnuScale64_Click(ByVal sender As Object, _
	 ByVal e As System.EventArgs) Handles mnuScale64.Click
		mnuThemeTopo.Enabled = True
		mnuLandmarks.Enabled = False
		strScale = "64"
		tsScale = TS.Scale.Scale64m
		Call ClearScaleCheck()
		mnuScale64.Checked = True
		Call DeleteSavedFiles()
		Call UpdateFromMenu()
	End Sub

	Private Sub mnuScale128_Click(ByVal sender As Object, _
	 ByVal e As System.EventArgs) Handles mnuScale128.Click
		mnuThemeTopo.Enabled = True
		mnuLandmarks.Enabled = False
		strScale = "128"
		tsScale = TS.Scale.Scale128m
		Call ClearScaleCheck()
		mnuScale128.Checked = True
		Call DeleteSavedFiles()
		Call UpdateFromMenu()
	End Sub

	Private Sub mnuScale256_Click(ByVal sender As Object, _
	 ByVal e As System.EventArgs) Handles mnuScale256.Click
		mnuThemeTopo.Enabled = True
		mnuLandmarks.Enabled = False
		strScale = "256"
		tsScale = TS.Scale.Scale256m
		Call ClearScaleCheck()
		mnuScale256.Checked = True
		Call DeleteSavedFiles()
		Call UpdateFromMenu()
	End Sub

	Private Sub ClearScaleCheck()
		'Clear any check mark
		mnuScale1.Checked = False
		mnuScale2.Checked = False
		mnuScale4.Checked = False
		mnuScale8.Checked = False
		mnuScale16.Checked = False
		mnuScale32.Checked = False
		mnuScale64.Checked = False
		mnuScale128.Checked = False
		mnuScale256.Checked = False
	End Sub

	'***********************************************
	'Pixel Format menu handlers
	'Pixel format only affects display and file size
	'***********************************************

	Private Sub mnuPixel24RGB_Click(ByVal sender As System.Object, _
	 ByVal e As System.EventArgs) Handles mnuPixel24RGB.Click
		mnuThemeTopo.Enabled = True
		pixFormat = PixelFormat.Format24bppRgb
		mnuPixel24RGB.Checked = True
		mnuPixel16RGB.Checked = False
		Call DeleteSavedFiles()
		Call UpdateDisplay(dblLatNav, dblLonNav)
	End Sub

	Private Sub mnuPixel16RGB_Click(ByVal sender As System.Object, _
	 ByVal e As System.EventArgs) Handles mnuPixel16RGB.Click
		mnuThemeTopo.Enabled = True
		pixFormat = PixelFormat.Format16bppRgb555
		mnuPixel24RGB.Checked = False
		mnuPixel16RGB.Checked = True
		Call DeleteSavedFiles()
		Call UpdateDisplay(dblLatNav, dblLonNav)
	End Sub

	'****************************
	'Navation Menu Event Handlers
	'****************************

	Private Sub mnuNavNorth_Click(ByVal sender As System.Object, _
	 ByVal e As System.EventArgs) Handles mnuNavNorth.Click
		'Move one tile north
		If blnIsMapPoint Then
			If dblLatNav = 0 Then
				dblLatNav = dblLatPoint
				dblLonNav = dblLonPoint
			End If
			dblLatNav = dblLatNav + GetLatNav()
			dblLonNav = dblLonNav
			bmpImage = GetMapPointImage(dblLatNav, dblLonNav, 640, 480)
			MyBase.Invalidate()
		Else
			dblLatNav = dblLatPoint + GetLatNav()
			dblLonNav = dblLonPoint
			Call UpdateDisplay(dblLatNav, dblLonNav)
		End If
	End Sub

	Private Sub mnuNavEast_Click(ByVal sender As System.Object, _
	 ByVal e As System.EventArgs) Handles mnuNavEast.Click
		'Move one tile east
		If blnIsMapPoint Then
			If dblLatNav = 0 Then
				dblLatNav = dblLatPoint
				dblLonNav = dblLonPoint
			End If
			dblLatNav = dblLatNav
			dblLonNav = dblLonNav - GetLonNav()
			bmpImage = GetMapPointImage(dblLatNav, dblLonNav, 640, 480)
			MyBase.Invalidate()
		Else
			dblLatNav = dblLatPoint
			dblLonNav = dblLonPoint - GetLonNav()
			Call UpdateDisplay(dblLatNav, dblLonNav)
		End If
	End Sub

	Private Sub mnuNavSouth_Click(ByVal sender As System.Object, _
	 ByVal e As System.EventArgs) Handles mnuNavSouth.Click
		'Move one tile south
		If blnIsMapPoint Then
			If dblLatNav = 0 Then
				dblLatNav = dblLatPoint
				dblLonNav = dblLonPoint
			End If
			dblLatNav = dblLatNav - GetLatNav()
			dblLonNav = dblLonNav
			bmpImage = GetMapPointImage(dblLatNav, dblLonNav, 640, 480)
			MyBase.Invalidate()
		Else
			dblLatNav = dblLatPoint - GetLatNav()
			dblLonNav = dblLonPoint
			Call UpdateDisplay(dblLatNav, dblLonNav)
		End If
	End Sub

	Private Sub mnuNavWest_Click(ByVal sender As System.Object, _
	 ByVal e As System.EventArgs) Handles mnuNavWest.Click
		'Move one tile west
		If blnIsMapPoint Then
			If dblLatNav = 0 Then
				dblLatNav = dblLatPoint
				dblLonNav = dblLonPoint
			End If
			dblLatNav = dblLatNav
			dblLonNav = dblLonNav + GetLonNav()
			bmpImage = GetMapPointImage(dblLatNav, dblLonNav, 640, 480)
			MyBase.Invalidate()
		Else
			dblLatNav = dblLatPoint
			dblLonNav = dblLonPoint + GetLonNav()
			Call UpdateDisplay(dblLatNav, dblLonNav)
		End If
	End Sub

	Private Sub mnuNavOrigin_Click(ByVal sender As System.Object, _
	 ByVal e As System.EventArgs) Handles mnuNavOrigin.Click
		'Return to origin
		blnIsFromAddress = False
		dblLatNav = 0
		dblLonNav = 0
		If blnIsMapPoint Then
			bmpImage = GetMapPointImage(dblLatNav, dblLonNav, 640, 480)
			MyBase.Invalidate()
		Else
			Call UpdateDisplay(dblLatNav, dblLonNav)
		End If
	End Sub

	Private Function GetLatNav() As Double
		'Calculate latitude difference per tile for navigation
		'by one tile in all directions
		Dim intDiv As Integer
		Select Case intTiles
			Case 12
				intDiv = 3
			Case 15
				intDiv = 3
			Case 16
				intDiv = 4
			Case 20
				intDiv = 4
			Case Else
				'Prevent divide by 0
				intDiv = 3
		End Select
		Return (dblLatNW - dblLatSE) / intDiv
	End Function

	Private Function GetLonNav() As Double
		'Calculate longitude difference per tile
		Dim intDiv As Integer

		Select Case intTiles
			Case 12
				intDiv = 4
			Case 15
				intDiv = 5
			Case 16
				intDiv = 4
			Case 20
				intDiv = 5
			Case Else
				'Prevent divide by 0
				intDiv = 4
		End Select
		Return (dblLonNW - dblLonSE) / intDiv
	End Function

	'*************************
	'Places List Form Handlers
	'*************************

	Private Sub mnuPlacesSel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuPlacesSel.Click
		'Display Places in Selection form
		blnIsFromPlaces = True
		blnEnterPlace = False
		frmPlaces.ShowDialog()
	End Sub

	Private Sub mnuPlacesNew_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuPlacesNew.Click
		'Display New Places Selection form variation
		blnIsFromPlaces = True
		blnEnterPlace = True
		frmPlaces.ShowDialog()
	End Sub

	'*********************
	'Landmarks Web Service
	'*********************

	Private Sub mnuLandmarksShow_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuLandmarksShow.Click
		frmLandmarks.Show()
	End Sub

	'*******************************
	'Return from Places or Landmarks
	'*******************************

	Private Sub TerraService_GotFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.GotFocus
		'Handles return from Places and Landmarks forms
		If blnIsFromPlaces Then
			mnuThemeTopo.Enabled = True
			'Load the selected image or map
			blnIsFromPlaces = False
			blnIsFromLandmarks = False
			blnIsFromAddress = False
			If InStr(Me.Text, strCity) = 0 Then
				Call DeleteSavedFiles()
				Call ClearScaleCheck()
				'Guess at appropriate scale for image
				If InStr(strCity, "Airport") > 0 Then
					mnuScale8.Checked = True
					tsScale = TS.Scale.Scale8m
					strScale = "8"
				ElseIf InStr(strCity, " Bay") > 0 Then
					mnuScale16.Checked = True
					tsScale = TS.Scale.Scale16m
					strScale = "16"
				Else
					mnuScale4.Checked = True
					tsScale = TS.Scale.Scale4m
					strScale = "4"
				End If
				dblLatNav = 0
				dblLonNav = 0
				Call UpdateFromMenu()
			End If
		ElseIf blnIsFromLandmarks Then
			blnIsFromAddress = False
			Call ClearScaleCheck()
			If InStr(strCity, "Populated Place") > 0 Then
				mnuScale4.Checked = True
				tsScale = TS.Scale.Scale4m
				strScale = "4"
			ElseIf InStr(strCity, "Building") > 0 Then
				mnuScale2.Checked = True
				tsScale = TS.Scale.Scale2m
				strScale = "2"
			Else
				mnuScale2.Checked = True
				tsScale = TS.Scale.Scale2m
				strScale = "2"
			End If
			dblLatNav = dblLatPoint
			dblLonNav = dblLonPoint
			If InStr(strCity, "(") > 0 Then
				Dim strTemp As String
				'Test for landmark type repeated in name and remove
				strTemp = Mid(strCity, InStr(strCity, "(") + 1)
				strTemp = Mid(strTemp, 1, InStr(strTemp, ")") - 1)
				If InStr(strCity, " " + strTemp) > 0 Then
					strCity = Replace(strCity, " (" + strTemp + ")", "")
				End If
			End If
			Me.Text = "TerraServer-USA Landmarks - " + strCity + _
			 " - Scale = " + strScale + "m/px"
			Call UpdateDisplay(dblLatNav, dblLonNav)
		End If
	End Sub

	'********************
	'Address Menu Handler
	'********************

	Private Sub mnuAddressEnter_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuAddressEnter.Click
		frmAddress.ShowDialog()
		If frmAddress.DialogResult = DialogResult.OK Then
			If lblInit.Visible Then
				'Test to prevent flashing
				lblInit.Visible = False
			End If
			Application.DoEvents()
			blnIsMapPoint = True
			blnIsFromAddress = True
			strScale = "2"
			Call ClearScaleCheck()
			mnuScale2.Checked = True
			tsScale = TS.Scale.Scale2m
			If Not (blnIsFromLandmarks Or blnIsFromPlaces) Then
				Me.Text = "MapPoint Address: " + strAddress + " (Scale = " + _
				 strScale + " m/px)"
			End If
			dblLatNav = 0
			dblLonNav = 0
			bmpImage = GetMapPointImage(dblLatPoint, dblLonPoint, 640, 480)
			If mnuThemeSaveFiles.Checked Then
				bmpImage.Save(strMapPointFile)
				blnMapPointFile = True
			End If
			'Draw the image
			MyBase.Invalidate()
			mnuTheme.Enabled = True
			mnuScale.Enabled = True
			mnuPixel.Enabled = True
			mnuThemePhoto.Checked = False
			mnuThemeTopo.Checked = False
			mnuThemeMapPoint.Checked = True
			blnIsFromLandmarks = False
			blnIsFromPlaces = False
			If intTiles = 0 Then
				'Address used first
				mnuNav.Enabled = False
			End If
		End If
	End Sub

	'**********************************
	'Links and Help Menu Handlers
	'**********************************

	Private Sub mnuTSHome_Click(ByVal sender As System.Object, _
	 ByVal e As System.EventArgs) Handles mnuTSHome.Click
		'Display TS home page
		Shell(strIE + " http://www.terraserver-usa.com", AppWinStyle.NormalFocus)
	End Sub

	Private Sub mnuTSWebServices_Click(ByVal sender As System.Object, _
	 ByVal e As System.EventArgs) Handles mnuTSWebServices.Click
		'Display TS Web Services page
		Shell(strIE + " http://www.terraserver-usa.com/webservices.aspx", _
		 AppWinStyle.NormalFocus)
	End Sub

	Private Sub mnuTSMP_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuTSMP.Click
		'Display MapPoint home page
		Shell(strIE + " http://www.microsoft.com/mappoint/default.mspx", _
		 AppWinStyle.NormalFocus)
	End Sub

	Private Sub mnuTSMPWS_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuTSMPWS.Click
		'Display MapPoint Web services page
		Shell(strIE + " http://www.microsoft.com/mappoint/webservice/default.mspx", _
		 AppWinStyle.NormalFocus)
	End Sub

	Private Sub mnuHelpOnline_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuHelpOnline.Click
		'Display the online help file
		If File.Exists(strIE) Then
			Shell(strIE + " http://www.oakleaf.ws/terraservice/aboutts.aspx", AppWinStyle.NormalFocus)
		Else
			Dim strMsg As String = "Can't find Internet Explorer at expected " + _
			 " location'" + strIE + "'. Go to " + _
			 "http://www.oakleaf.ws/terraservice/aboutts.aspx" + _
			 " for online help with TerraService client features."
			MsgBox(strMsg, MsgBoxStyle.Information, "Internet Explorer Not Found")
		End If
	End Sub

	Private Sub mnuHelpAbout_Click(ByVal sender As Object, _
	 ByVal e As System.EventArgs) Handles mnuHelpAbout.Click
		'Display the About the Client form
		frmAbout.ShowDialog()
	End Sub

	'*******************************************
	'Display Image Properties form (right-click)
	'*******************************************

	Private Sub TerraService_MouseDown(ByVal sender As Object, _
	 ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseDown
		If e.Button = MouseButtons.Right And _
		 Not (bmpImage Is Nothing Or blnIsMapPoint) Then
			'Don't display image properties for images from files
			If (blnPhotoFile And Not blnIsTopoMap) Or (blnTopoFile And blnIsTopoMap) Then
				Dim strMsg As String = "Image data isn't available with locally-stored aerial photos or topo maps."
				MsgBox(strMsg, MsgBoxStyle.Information, "Image Data Not Available")
				Exit Sub
			End If
			If mnuFileImageProps.Enabled Then
				'Display the ImageData form
				frmImageData.ShowDialog()
			End If
		End If
	End Sub

	'************************************
	'Following used by several procedures
	'************************************

	Private Sub DeleteSavedFiles()
		'Delete the saved theme files
		Application.DoEvents()
		File.Delete(strPhotoFile)
		File.Delete(strTopoFile)
		File.Delete(strMapPointFile)
		blnPhotoFile = False
		blnTopoFile = False
		blnMapPointFile = False
	End Sub

	'***********************************
	'Copy Image to the Windows Clipboard
	'***********************************

	Private Sub TerraService_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles MyBase.KeyDown
		If e.Control And e.KeyCode = Keys.C And Not bmpImage Is Nothing Then
			'Save the current image to the clipboard
			Clipboard.SetDataObject(bmpImage, True)
		End If
	End Sub
End Class
